// Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 2430 $
// $LastChangedDate:: 2022-01-26 20:01:55 +0100#$
//
// Creates C-code computational function of IO blocks in General library.

// disables warnings for this function to avoid 'identifier truncated' warnings for long (local) function names (required for Scilab 5.x only)
cur_warnmode = warning('query');
warning('off');

function [] = createCFunctionIO()

// import necessary java classes
jimport at.lcm.x2c.utils.LibraryUtils;

// get file separator
fs = filesep(); 

// get some block information
libName = "General";
xmlPath = jinvoke(LibraryUtils, "getXmlDirectory", libName);
xmlDir = jinvoke(xmlPath, "toString");
blockInfo.scilabDir = strsubst(xmlDir, ["XML" + fs], ["Scilab" + fs]);

// get datatype names of IO-blocks
ImplNames = getIODataTypes();

//------------------------------------------------------------------------------
// Create C-functions for inport
//------------------------------------------------------------------------------
curXml = [xmlPath + "Inport.xml"];
blockInfo.blockName = "Inport";
blockInfo.xcosName = "x2c_Inport";
try 
    // go through all available implementations
    for curImpl=ImplNames
        
        // open/create file
        fileName = blockInfo.scilabDir + blockInfo.xcosName + "_" + curImpl + "_C.c";
        fileID = mopen(fileName, "wt");
        
        // print header
        writeHeader(fileID, blockInfo.blockName, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");
        
        writeSimulationFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        //----------------------------------------------------------------------
        // Initialization
        //----------------------------------------------------------------------
        writeInitFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        //----------------------------------------------------------------------
        // Ending
        //----------------------------------------------------------------------
        writeEndingFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        //----------------------------------------------------------------------
        // State Update f
        //----------------------------------------------------------------------
        writeStateUpdateLocalFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");
        
        //----------------------------------------------------------------------
        // State Update
        //----------------------------------------------------------------------
        writeStateUpdateFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");
        
        //----------------------------------------------------------------------
        // Output Update
        //----------------------------------------------------------------------
        writeOutputUpdateFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        // close file
        mclose(fileID);
        mprintf("Computational function %s successfully written.\n", fileName);
    end
    
catch
    // some error creating the file occured
    //mclose(fileID); // try to close the file
    disp("Error creating C-code files for block " + blockInfo.blockName + ": " + lasterror());
end

//------------------------------------------------------------------------------
// Create C-code computational functions for outport
//------------------------------------------------------------------------------
curXml = [xmlPath + "Outport.xml"];
blockInfo.blockName = "Outport";
blockInfo.xcosName = "x2c_Outport";
try 
    // go through all available implementations
    for curImpl=ImplNames

        // open/create file
        fileName = blockInfo.scilabDir + blockInfo.xcosName + "_" + curImpl + "_C.c";
        fileID = mopen(fileName, "wt");
        
        // print header
        writeHeader(fileID, blockInfo.blockName, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");
        
        writeSimulationFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        //----------------------------------------------------------------------
        // Initialization
        //----------------------------------------------------------------------
        writeInitFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        //----------------------------------------------------------------------
        // Ending
        //----------------------------------------------------------------------
        writeEndingFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        //----------------------------------------------------------------------
        // State Update f
        //----------------------------------------------------------------------
        writeStateUpdateLocalFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");
        
        //----------------------------------------------------------------------
        // State Update
        //----------------------------------------------------------------------
        writeStateUpdateFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");
        
        //----------------------------------------------------------------------
        // Output Update
        //----------------------------------------------------------------------
        writeOutputUpdateFunction(fileID, blockInfo.xcosName, curImpl);
        mfprintf(fileID, "\n\n");

        // close file
        mclose(fileID);
        mprintf("Computational function %s successfully written.\n", fileName);
    end
    
catch
    // some error creating the file occured
    //mclose(fileID); // try to close the file
    disp("Error creating C-code files for block " + blockInfo.blockName + ": " + lasterror());
end

endfunction


function writeHeader(file_id, block_name, xcos_block_name, imp_name)
        time = datevec(datenum());
        mfprintf(file_id, "// This file was generated by %s on %02d-%02d-%04d %02d:%02d\n\n", "createCFunctionIO.sci", time(3), time(2), time(1), time(4), time(5));
        mfprintf(file_id, "// Computational function for X2C block %s\n\n", block_name);
        // TODO: expand file header (e.g. with license)

        mfprintf(file_id, "#define SCILAB_SIM_FILE\n");
        mfprintf(file_id, "\n");
        mfprintf(file_id, "/* include Scicos / Xcos headers */\n");
        mfprintf(file_id, "#include <scicos.h>\n");
        mfprintf(file_id, "#include <scicos_block4.h>\n");
        mfprintf(file_id, "#include <scicos_malloc.h>\n");
        mfprintf(file_id, "#include <scicos_free.h>\n");
        mfprintf(file_id, "#include <Simulation_PortConversion.h>\n");
        mfprintf(file_id, "\n");
        mfprintf(file_id, "/* public prototypes */\n");
        mfprintf(file_id, "void %s_%s_C(scicos_block* block, scicos_flag flag);\n", xcos_block_name, imp_name);
        mfprintf(file_id, "/* private prototypes */\n");
        mfprintf(file_id, "static void %s_%s_C__OutputUpdate(scicos_block* block);\n", xcos_block_name, imp_name);
        mfprintf(file_id, "static void %s_%s_C__StateUpdate(scicos_block* block);\n", xcos_block_name, imp_name);
        mfprintf(file_id, "static void %s_%s_C__Initialization(scicos_block* block);\n", xcos_block_name, imp_name);
        mfprintf(file_id, "static void %s_%s_C__Ending(scicos_block* block);\n", xcos_block_name, imp_name);
endfunction


function writeInitFunction(file_id, xcos_block_name, imp_name)
    mfprintf(file_id, "static void %s_%s_C__Initialization(scicos_block* block) {\n", xcos_block_name, imp_name);
    mfprintf(file_id, "    int *dummy;\n");
    mfprintf(file_id, "    dummy = scicos_malloc(sizeof(int));\n");
    mfprintf(file_id, "    GetWorkPtrs(block) = dummy;\n");
    mfprintf(file_id, "}\n");
endfunction


function writeEndingFunction(file_id, xcos_block_name, imp_name)
    mfprintf(file_id, "static void %s_%s_C__Ending(scicos_block* block) {\n", xcos_block_name, imp_name);
    mfprintf(file_id, "    int *dummy;\n");
    mfprintf(file_id, "    dummy = GetWorkPtrs(block);\n");
    mfprintf(file_id, "    scicos_free(dummy);\n");
    mfprintf(file_id, "}\n");
endfunction


function writeStateUpdateFunction(file_id, xcos_block_name, imp_name)
    mfprintf(file_id, "static void %s_%s_C__StateUpdate(scicos_block* block) {\n", xcos_block_name, imp_name);
    mfprintf(file_id, "    if (GetNevIn(block) > 0) {\n");
    mfprintf(file_id, "        /*\n");
    mfprintf(file_id, "        * GetNevIn(block) returns -1 in in case this function was called due to an internal zero-crossing.\n");
    mfprintf(file_id, "        * GetNevIn(block)  ... activation index\n");
    mfprintf(file_id, "        */\n");
    mfprintf(file_id, "        %s_%s_C__StateUpdate_f(block);\n", xcos_block_name, imp_name);
    mfprintf(file_id, "    }\n");
    mfprintf(file_id, "}\n");
endfunction


function writeStateUpdateLocalFunction(file_id, xcos_block_name, imp_name)
    mfprintf(file_id, "static void %s_%s_C__StateUpdate_f(scicos_block* block) {\n", xcos_block_name, imp_name);
    mfprintf(file_id, "    double *outport_Out;\n");
    mfprintf(file_id, "    double *inport_In_p;\n");
    mfprintf(file_id, "    \n");
    mfprintf(file_id, "    outport_Out = GetOutPortPtrs(block, 1);\n");
    mfprintf(file_id, "    inport_In_p = GetInPortPtrs(block, 1);\n");
    cur_format = dataTypeToQFormat(imp_name);
    mfprintf(file_id, "    *outport_Out = convert_%s_to_double(convert_double_to_%s(*inport_In_p));\n", cur_format, cur_format);
    mfprintf(file_id, "}\n");
endfunction


function writeSimulationFunction(file_id, xcos_block_name, imp_name)
    mfprintf(file_id, "void %s_%s_C(scicos_block* block, scicos_flag flag) {\n", xcos_block_name, imp_name);
    mfprintf(file_id, "    /*\n");
    mfprintf(file_id, "     * This function will be called by Xcos\n");
    mfprintf(file_id, "     */\n");
    mfprintf(file_id, "    switch (flag) {\n");
    mfprintf(file_id, "        case DerivativeState:  /* 0 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "            break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        case OutputUpdate:     /* 1 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "            %s_%s_C__OutputUpdate(block);\n", xcos_block_name, imp_name);
    mfprintf(file_id, "            break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        case StateUpdate:      /* 2 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "            %s_%s_C__StateUpdate(block);\n", xcos_block_name, imp_name);
    mfprintf(file_id, "            break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        case OutputEventTiming: /* 3 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "            break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        case Initialization:   /* 4 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "            %s_%s_C__Initialization(block);\n", xcos_block_name, imp_name);
    mfprintf(file_id, "            break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        case Ending:           /* 5 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "            %s_%s_C__Ending(block);\n", xcos_block_name, imp_name);
    mfprintf(file_id, "            break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        case ReInitialization: /* 6 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "            break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        case ZeroCrossing:     /* 9 */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "           break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "        default:     /* ContinousPropertiesUpdate (v5.5.x) Jacobian (v5.5.x) or Residute (v5.4.1) or something else */\n");
    mfprintf(file_id, "        {\n");
    mfprintf(file_id, "           break;\n");
    mfprintf(file_id, "        }\n");
    mfprintf(file_id, "    }\n");
    mfprintf(file_id, "}\n");
endfunction


function writeOutputUpdateFunction(file_id, xcos_block_name, imp_name)
    mfprintf(file_id, "static void %s_%s_C__OutputUpdate(scicos_block* block) {\n", xcos_block_name, imp_name);
    mfprintf(file_id, "    /* Time dependency -> execute Output assignment in State Update function */\n");
    mfprintf(file_id, "    ;\n");
    mfprintf(file_id, "}\n");
endfunction


// Converts enum data type string to corresponding Q-format string
// Parameters:
// dataType - enum data type
// QFormat - corresponding Q-format string
function [QFormat] = dataTypeToQFormat(dataType)
    
select dataType
    case "int8" then
        QFormat = "Q7";
    case "uint8" then
        QFormat = "Q7";
    case "int16" then
        QFormat = "Q15";
    case "uint16" then
        QFormat = "Q15";
    case "int32" then
        QFormat = "Q31";
    case "uint32" then
        QFormat = "Q31";
    case "int64" then
        QFormat = "Q63";
    case "uint64" then
        QFormat = "Q63";
    case "float32" then
        QFormat = "float32";
    case "float64" then
        QFormat = "float64";
    else
        QFormat = dataType;
end
    
endfunction

// set warning enable state to previous state
warning(cur_warnmode);
clear("cur_warnmode");
