// Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 2495 $
// $LastChangedDate:: 2022-02-28 16:21:19 +0100#$
//
// Function to create 'starter.sce' for X2C libraries
// Parameters:
// libName - Library name
// varargin - (optional) project root directory for external blocks

function [] = createLibraryStarter(libName, varargin)

// import necessary java classes
jimport at.lcm.x2c.core.structure.ControlBlock;
jimport at.lcm.x2c.core.structure.Block;
jimport at.lcm.x2c.utils.LibraryUtils;
jimport at.lcm.x2c.utils.Utils;
jimport at.lcm.x2c.core.structure.VisualizationScilabXcos;

// get current directory
currentDir = pwd();

// more or less arbitrary chosen suffix for palette
paletteSuffix = ".xpal";

// check library type
if length(varargin) == 0 then
    // internal library
    isIntern = %t;
elseif length(varargin) == 1 then
    // external library
    isIntern = %f;
else
    error("Invalid number of input arguments");
end

// get XML file names
if isIntern then
    // internal library
    xmlPath = jinvoke(LibraryUtils, "getXmlDirectory", libName);
    libRoot = jinvoke(jinvoke(LibraryUtils, "getLibraryRootDirectory"), "toString");
else
    // external library
    libRoot = jinvoke(jinvoke(LibraryUtils, "getLibraryRootDirectory", varargin(1)), "toString");
    xmlPath = jinvoke(LibraryUtils, "getXmlDirectory", libName, libRoot);
end
xmlDir = jinvoke(xmlPath, "toString");

// library starter script is always located in the 'Scilab' directory (used by all supported Scilab versions)
scilabDir = fullfile(libRoot, libName, "Scilab");
// create library starter directory if it does not exist
if ~isdir(scilabDir) then
    mkdir(scilabDir)
end

// get XML files
xmlFiles = dir(xmlDir + "*.xml");
xmlFiles = xmlFiles(2)';
xmlList = [];

// go through XML list
for curXML = xmlFiles
    if ~strcmp(part(curXML, $-3:$), ".xml", "i") then
        // add file name with *.xml ending
        xmlList = [xmlList, curXML];
    end
end

chdir(scilabDir);
// get file relevant file names
interfaceList = ls("*.sci")';

// remove palette
palette_file_name = [libName + paletteSuffix];
if isfile(palette_file_name) then
    mdelete(palette_file_name);
end
chdir(currentDir);

try
    // open/create file
    fileName = fullfile(scilabDir, "starter.sce");
    fileID = mopen(fileName, "wt");
    
    // print header
    time = datevec(datenum());
    mfprintf(fileID, "// This file was generated by %s on %02d-%02d-%04d %02d:%02d\n\n", "createLibraryStarter.sce", time(3), time(2), time(1), time(4), time(5));
    // TODO: expand file header
    
    mfprintf(fileID, "mprintf('"+++ Loading X2C library %s\\n'");\n", libName); 
    mfprintf(fileID, "scilab_dir = getLibraryScilabPath(fullfile(get_absolute_file_path('"starter.sce'"), '"..'"));\n");
    mfprintf(fileID, "\n");
    
    // enter library help
    mfprintf(fileID, "// load and add help chapter\n");
    mfprintf(fileID, "if or(getscilabmode() == ['"NW'";'"STD'"]) then\n");
    mfprintf(fileID, "    path_addchapter = fullfile(scilab_dir, '"jar'");\n");
    mfprintf(fileID, "    if isdir(path_addchapter) then\n");
    mfprintf(fileID, "        add_help_chapter('"X2C %s Library'", path_addchapter, %%F);\n", libName);
    mfprintf(fileID, "    end\n");
    mfprintf(fileID, "end\n");
    mfprintf(fileID, "\n");
    
    // enter all interface functions
    mfprintf(fileID, "// load interface functions\n");
    for curInterface = interfaceList
        mfprintf(fileID, "exec('"%s'", -1);\n", curInterface);
    end
    mfprintf(fileID, "\n");
    
    // palette file in correct Scilab directory
    mfprintf(fileID, "palette_file = fullfile(scilab_dir, '"%s'");\n", palette_file_name);
    
    // enter all blocks
    mfprintf(fileID, "// load/create palette for library\n");
    mfprintf(fileID, "if isfile(palette_file) then\n");
    mfprintf(fileID, "    // palette already exists -> load saved palette\n");
    mfprintf(fileID, "    xcosPalAdd(palette_file, '"X2C'");\n");
    mfprintf(fileID, "else\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    // load blocks and build palette\n");
    mfprintf(fileID, "    pal = xcosPal('"%s'");\n", libName);
    mfprintf(fileID, "    allIconsAvailable = %%t;\n");
    mfprintf(fileID, "    \n");
    
    mfprintf(fileID, "    icon_dir = fullfile(scilab_dir, '"Icons'");\n");
    mfprintf(fileID, "    \n");
    
    // add normal blocks
    for curXml=xmlList

        // read block XML file
        x2cBlock = jinvoke(Block, "loadBlockXML", curXml);
        blockType = jinvoke(jinvoke(x2cBlock, "getClass"), "toString");
        blockName = jinvoke(x2cBlock, "getName");
        
        // check block type
        if ~strcmp(blockType, "class at.lcm.x2c.core.structure.ControlBlock") then
            // Control Block
            if isIntern then
                // internal library
                xcosName = getXcosBlockName(libName, blockName);
            else
                // external library
                xcosName = getXcosBlockName(libName, blockName, varargin(1));
            end
            
            // enter block data
            mfprintf(fileID, "    // %s\n", blockName);
            mfprintf(fileID, "    mprintf('"Loading block %s...'");\n", blockName);
            mfprintf(fileID, "    o = %s('"define'");\n", xcosName);
            mfprintf(fileID, "    pal_icon = fullfile(icon_dir, '"%s.png'");\n", blockName);
            mfprintf(fileID, "    if ~isfile(pal_icon) then\n")
            mfprintf(fileID, "        pal_icon = [];\n");
            mfprintf(fileID, "        allIconsAvailable = %%f;\n");
            mfprintf(fileID, "    end\n");
        else
            // IO Block
            mfprintf(fileID, "    // %s\n", blockName);
            mfprintf(fileID, "    mprintf('"Loading block %s...'");\n", blockName);
            mfprintf(fileID, "    o = x2c_%s('"define'");\n", blockName);
            mfprintf(fileID, "    pal_icon = fullfile(icon_dir, '"%s.png'");\n", blockName);
            mfprintf(fileID, "    if ~isfile(pal_icon) then\n")
            mfprintf(fileID, "        pal_icon = [];\n");

            mfprintf(fileID, "        allIconsAvailable = %%f;\n");
            mfprintf(fileID, "    end\n");
        end
        
        mfprintf(fileID, "    pal = xcosPalAddBlock(pal, o, pal_icon, o.graphics.style);\n");
        mfprintf(fileID, "    clear('"o'");\n");
        mfprintf(fileID, "    mprintf('"done\\n'");\n");
        mfprintf(fileID, "    \n");
    end
    
    mfprintf(fileID, "    \n\n");
    mfprintf(fileID, "    // load palette\n");
    mfprintf(fileID, "    xcosPalAdd(pal, '"X2C'");\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    // save palette if all icons are available\n");
    mfprintf(fileID, "    if allIconsAvailable then\n");
    mfprintf(fileID, "        // create directory for XPAL file if it does not exist\n");
    mfprintf(fileID, "        if ~isdir(scilab_dir) then\n");
    mfprintf(fileID, "            mkdir(scilab_dir);\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        xcosPalExport(pal, palette_file);\n");
    mfprintf(fileID, "    end\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "end\n");
    mfprintf(fileID, "\n");
    
    mfprintf(fileID, "// cleanup\n");
    mfprintf(fileID, "clear('"pal'", '"pal_icon'", '"style'", '"allIconsAvailable'", '"icon_dir'", '"scilab_dir'");\n");
    mfprintf(fileID, "clear('"path_addchapter'", '"palette_file'");\n");
    mfprintf(fileID, "\n");
    
    mfprintf(fileID, "mprintf('"---\\n'");\n"); 

    // close file
    mclose(fileID);
    mprintf("Library starter file %s successfully written.\n", fileName);
catch
    // some error creating the file occured
    mclose(fileID); // try to close the file
    mprintf("Error creating starter file for library " + libName + ": " + lasterror() + "\n");
end

endfunction
